#include <ESP8266WiFi.h>          

//needed for library
#include <DNSServer.h>
#include <ESP8266WebServer.h>
#include <ESP8266mDNS.h>
#include <WiFiManager.h>          
ESP8266WebServer server(80);

//for LED status
#include <Ticker.h>
Ticker ticker;

static const uint8_t IN_LED = 2;
static const char* MDSN_NAME = "logger";
   
unsigned long firsttime = 0;          // when the ball goes into the first sensor
unsigned long firsttime2 = 0;         // when the ball leaves the first sensor
unsigned long secondtime = 0;         // when the ball goes into the second sensor
unsigned long secondtime2 = 0;        // when the ball leaves the second sensor

unsigned long difference = 0;         // the time while the ball goes through the first sensor
unsigned long difference2 = 0;        // the time while the ball goes through the second sensor
float ballSpeed = 0;                  // the first speed
float ballSpeed2 = 0;                 // the second speed
float acceleration = 0;               // the acceleration of the ball
float ballsize = 0;                   // the size of the ball     
int buttonState = 0;

int IRval1 = 0;
int IRval2 = 0;

// first IR input is in the D2, which is the pin 5
// second IR inpput is in the D1, which is the pin 16
// the IR output (power) is in the D3, which is the pin 4 
// the button is in the D5, which is the pin 14
// blue indicator led (for wifi) is in the D8, which is the pin 15
// red indicator led (for the right sensor) is in the D7, which is the pin 13
// yellow indicator led (for the left sensor) is int the D6, which is the pin 12

const int irLed1 = 0;
const int irLed2 = 15;
const int inputPin1 = 5;
const int inputPin2 = 16;

const int blueIndicator = 2;
const int redIndicator = 13;
const int yellowIndicator = 12;

const int buttonPin = 14;

enum htmlState_t {          // set the enums to the each states   
  MAIN, 
  PA,
  MERES2,
  MERES3,
  INPROGRESS,
  PAEREDMENY
} htmlState;

enum meresState_t {          // set the enums to the each states   
  NO,
  PA_MERES,
  MASODIK,
  HARMADIK
} meresState;

enum plainAccelerationState_t {   // the enum for the first experiment, the plain acceleration
  PA_NO,
  PA_MERES1,
  PA_MERES2,
  PA_MERES3,
  PA_MERES4,
  PA_CALC
} plainAccelerationState;

void tick()
{
  //toggle state
  int state = digitalRead(IN_LED);  // get the current state of GPIO1 pin
  digitalWrite(IN_LED, !state);     // set pin to the opposite state
}

void leds(){
  if(IRval2 == 0){
    digitalWrite(yellowIndicator, HIGH);
  }
  else{
    digitalWrite(yellowIndicator, LOW);
  }
  if(IRval1 == 0){
    digitalWrite(redIndicator, HIGH);
  }
  else{
    digitalWrite(redIndicator, LOW);
  }
}

String ipToString(IPAddress ip){
  String s="";
  for (int i=0; i<4; i++)
    s += i  ? "." + String(ip[i]) : String(ip[i]);
  return s;
}

//gets called when WiFiManager enters configuration mode
void configModeCallback (WiFiManager *myWiFiManager) {
  Serial.println("Entered config mode");
  Serial.println(WiFi.softAPIP());
  //if you used auto generated SSID, print it
  Serial.println(myWiFiManager->getConfigPortalSSID());
  //entered config mode, make led toggle faster
  ticker.attach(0.2, tick);
}

String mainHtml() {
  String message = "<!DOCTYPE html>\n";
  message += "<html><head>\n";
  message += "<style>\n";
  message += "body {text-align: center;}   h1 {text-align: center;}    h2 {text-align:center;}  h5 {text-align:center; margin: 0px;}p {text-align: center;}\n";
  message += ".buttonA { background-color: #4CAF50;  border: none;  color: white;    padding: 15px 0px;  width: 100%; text-align: center;  text-decoration: none;  display: inline-block;  cursor: pointer; margin: 0px;}\n";
  message += ".buttonB { background-color: #ff9900;  border: none;  color: white;    padding: 15px 0px;  width: 100%; text-align: center;  text-decoration: none;  display: inline-block;  cursor: pointer;}\n";
  message += "</style>\n";
  message += "<meta charset='utf-8'>\n";
  message += "<title>[logger]</title>\n";
  message += "<meta name='viewport' content='width=device-width, initial-scale=1.0'>\n";
  message += "</head><body>\n";
  message += "<h1 class='buttonB'>[logger]</h1>\n";
  message += "<h5>"+ipToString(WiFi.localIP())+"</h5>\n";
  message += "<h2>What do you want to do?</h2>\n";
  message += "<h3>Calculate...</h3>\n";
  message += "<p><a href='/PA' class='buttonA'>Plain acceleration</a></p>\n";
  message += "<p><a href='/meres2' class='buttonA'>Gradient acceleration</a></p>\n";
  message += "<p><a href='/meres3' class='buttonA'>Friction on different surfaces</a></p>\n";
  message += "<p><a href='http://simi.hu/experiments' class='buttonB'>About the experiments</a></p>\n";
  message += "<p><a href='http://simi.hu/logger' class='buttonB'>About the [logger]</a></p>\n";
  message += "</body></html>\n";
  return message;
}

String plainAccelerationHtml() {
  String message = "<!DOCTYPE html>\n";
  message += "<html><head>\n";
  message += "<style>\n";
  message += "body {text-align: center;}   h1 {text-align: center;}    h2 {text-align:center;}  h5 {text-align:center; margin: 0px;}p {text-align: center;}\n";
  message += ".buttonA { background-color: #4CAF50;  border: none;  color: white;    padding: 15px 0px;  width: 100%; text-align: center;  text-decoration: none;  display: inline-block;  cursor: pointer; margin: 0px;}\n";
  message += ".buttonB { background-color: #ff9900;  border: none;  color: white;    padding: 15px 0px;  width: 100%; text-align: center;  text-decoration: none;  display: inline-block;  cursor: pointer;}\n";
  message += "</style>\n";
  message += "<meta charset='utf-8'>\n";
  message += "<title>[logger]</title>\n";
  message += "<meta name='viewport' content='width=device-width, initial-scale=1.0'>\n";
  message += "</head><body>\n";
  message += "<h1 class='buttonB'>PLAIN ACCELERATION</h1>\n";
  message += "<form action='PAData' method='get' name='dataform'>\n";
  message += "<p><label for='ballsize'>Size of the ball:</label><input name='ballsize' type='number' id='ballsize' min='31' max='55' value='42'>mm</p>\n";
  message += "<h2><input type='button' class='buttonB' value='Start' onclick='document.dataform.submit();'></h2>\n";
  message += "<p><a href='/done' class='buttonA'>Menu</a></p>\n";
  message += "</form></body></html>\n";
  return message;
}

String PAEredmenyHtml() {
  meresState = NO;  
  String message = "<!DOCTYPE html>\n";
  message += "<html><head>\n";
  message += "<style>\n";
  message += "body {text-align: center;}   h1 {text-align: center;}    h2 {text-align:center;}  h5 {text-align:center; margin: 0px;}p {text-align: center;}\n";
  message += ".buttonB { background-color: #ff9900;  border: none;  color: white;    padding: 15px 0px;  width: 100%; text-align: center;  text-decoration: none;  display: inline-block;  cursor: pointer;}\n";
  message += "</style>\n";
  message += "<meta charset='utf-8'>\n";
  message += "<title>[logger]</title>\n";
  message += "<meta name='viewport' content='width=device-width, initial-scale=1.0'>\n";
  message += "</head><body>\n";
  message += "<h1 class='buttonB'>RESULST</h1>\n";
  message += "<p>First speed: " + String(ballSpeed) + " m/s </p>";
  message += "<p>Second speed: " + String(ballSpeed2) + " m/s </p>";
  message += "<p>The Acceleration: " + String(acceleration) + " m/s2 </p>";
  message += "<p><a href='/done' class='buttonB'>Done</a></p>\n";
  message += "</body></html>\n";
  return message;
}

String inProgressHtml() {
  String message = "<!DOCTYPE html>\n";
  message += "<html><head>\n";
  message += "<style>\n";
  message += "body {text-align: center;}   h1 {text-align: center;}    h2 {text-align:center;}  h5 {text-align:center; margin: 0px;}p {text-align: center;}\n";
  message += ".buttonB { background-color: #ff9900;  border: none;  color: white;    padding: 15px 0px;  width: 100%; text-align: center;  text-decoration: none;  display: inline-block;  cursor: pointer;}\n";
  message += "</style>\n";
  message += "<meta charset='utf-8'>\n";
  message += "<meta http-equiv='refresh' content='2'>\n";
  message += "<title>[logger]</title>\n";
  message += "<meta name='viewport' content='width=device-width, initial-scale=1.0'>\n";
  message += "</head><body>\n";
  message += "<h1 class='buttonB'>[logger]</h1>\n";
  message += "<h2>You can roll the ball now!</h2>\n";
  message += "</body></html>\n";
  return message;
}

void handlePAData(){
  if (server.hasArg("ballsize")){
    ballsize = server.arg("ballsize").toInt();
    htmlState = INPROGRESS;
    meresState = PA_MERES;
    plainAccelerationState = PA_MERES1;
    digitalWrite(irLed1, LOW);
    digitalWrite(irLed2, LOW);
    delay(500);
    String header = "HTTP/1.1 301 OK\r\nLocation: /\r\nCache-Control: no-cache\r\n\r\n";
    server.sendContent(header);
    return;
  }
}

void handlePA() {
  ticker.detach();
  htmlState = PA;
  String header = "HTTP/1.1 301 OK\r\nLocation: /\r\nCache-Control: no-cache\r\n\r\n";
  server.sendContent(header);
  return;  
}

void handleDone(){
  ticker.attach(1, tick);
  htmlState = MAIN;
  String header = "HTTP/1.1 301 OK\r\nLocation: /\r\nCache-Control: no-cache\r\n\r\n";
  server.sendContent(header);
  return;  
}

void handleRoot() {
  digitalWrite(IN_LED, HIGH);
  String pageData;
  switch(htmlState){
    case MAIN:
      pageData = mainHtml();
      break;
    case PA:
      pageData = plainAccelerationHtml();
      break;
    case INPROGRESS:
      pageData = inProgressHtml();
      break;
    case PAEREDMENY:
      pageData = PAEredmenyHtml();
      break;
      
    case MERES2:
      //pageData = meres2Html();
      break;
    case MERES3:
      //pageData = meres3Html();
      break;
 }
 server.send(200, "text/html", pageData);
 digitalWrite(IN_LED, LOW);
}

void handleNotFound(){
  digitalWrite(IN_LED, HIGH);
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += (server.method() == HTTP_GET)?"GET":"POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";
  for (uint8_t i=0; i<server.args(); i++){
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, "text/plain", message);
  digitalWrite(IN_LED, LOW);
}

void setup() {
  Serial.begin(115200); 
  
  pinMode(irLed1, OUTPUT);  
  pinMode(irLed2, OUTPUT); 
  pinMode(blueIndicator, OUTPUT);
  pinMode(redIndicator, OUTPUT);
  pinMode(yellowIndicator, OUTPUT);
  pinMode(buttonPin, INPUT);
  
  digitalWrite(irLed1, HIGH);
  digitalWrite(irLed2, HIGH);
  
  ticker.attach(0.4, tick);

  WiFiManager wifiManager;
  if (!wifiManager.autoConnect()) {
    Serial.println("failed to connect and hit timeout");
    ESP.reset();
    delay(1000);
  }

  Serial.println("connected...yeey :)");
  ticker.detach();
  digitalWrite(blueIndicator, LOW);
  ticker.attach(1, tick);

  if (!MDNS.begin(MDSN_NAME)) {
    Serial.println("Error setting up MDNS responder!");
    while(1) { 
      delay(1000);
    }
  }
  Serial.println("mDNS responder started");

  server.on("/", handleRoot);
  server.on("/PA", handlePA);
  server.on("/PAData", handlePAData);
  server.on("/done", handleDone);

  server.onNotFound(handleNotFound);

  server.begin();
  Serial.println("HTTP server started");

  MDNS.addService("http", "tcp", 80);
  
  htmlState = MAIN;
  meresState = NO;
  plainAccelerationState = PA_NO;
}

// experiments from here //

void updateValues (){
  IRval1 = digitalRead(inputPin1);
  IRval2 = digitalRead(inputPin2);
  buttonState = digitalRead(buttonPin);
}

void paMeres1(){          // the first measurement, when the ball reaches the first gate
  updateValues();
  if (IRval1 == 0){
    Serial.println("ball in the first gate");
    firsttime = micros();
    plainAccelerationState = PA_MERES2;
  }
}

void paMeres2(){          // the second measurement, when the ball leaves the first gate
  updateValues();
  if (IRval1 == 1){
    Serial.println("ball out of the first gate");
    firsttime2 = micros();
    plainAccelerationState = PA_MERES3;
  }
}

void paMeres3(){          // the first measurement, when the ball reaches the second gate
  updateValues();
  if (IRval2 == 0){
    Serial.println("3");
    secondtime = micros();
    plainAccelerationState = PA_MERES4;
  }
}

void paMeres4(){          // the second measurement, when the ball leaves the second gate
  updateValues();
  if (IRval2 == 1){
    Serial.println("4");
    secondtime2 = micros();
    plainAccelerationState = PA_CALC; 
  }
}

void paCalc() {
  digitalWrite(irLed1, HIGH);
  digitalWrite(irLed2, HIGH);
  
  Serial.println("calculating");
  difference = firsttime2 - firsttime;                        // calculating the times 
  ballSpeed = ballsize / difference * 1000;
  difference2 = secondtime2 - secondtime;
  ballSpeed2 = ballsize / difference2 * 1000;

  acceleration = (ballSpeed2 - ballSpeed) / (secondtime2 - firsttime2) * 1000000;           // calcualing the acceleration

  Serial.println("=========================================");                              // prints the the values to serial monitor
  Serial.print("ball size: ");
  Serial.println(ballsize);
  Serial.print("difference: ");
  Serial.println(difference);
  Serial.print("difference2: ");
  Serial.println(difference2);
  Serial.print("The speed was: ");
  Serial.print(ballSpeed);
  Serial.println(" m/s");
  Serial.print("The second speed was: ");
  Serial.print(ballSpeed2);
  Serial.println(" m/s");
  Serial.print("The acceleration was: ");
  Serial.print(acceleration);
  Serial.println(" m/s2");
  Serial.println("=========================================");
  plainAccelerationState = PA_NO;
  htmlState = PAEREDMENY;
}


void plainAccelerationMeres(){
  leds();
  switch(plainAccelerationState){
    case PA_MERES1:
    paMeres1();
    break;
    case PA_MERES2:
    paMeres2();
    break;
    case PA_MERES3:
    paMeres3();
    break;
    case PA_MERES4:
    paMeres4();
    break;
    case PA_CALC:
    paCalc();
    break;    
  }
}

void loop() {
  // put your main code here, to run repeatedly:
  server.handleClient();
  switch(meresState){
    case PA:                  //case to the plain accleration
      plainAccelerationMeres();
      break;
    case MASODIK:
      //meres2();
      break;
    case HARMADIK:
      //meres3();
      break;
 }
}
